//
// NV_transform_feedback vertex shader
//

varying vec4  newPos;
varying vec4  newVel;
uniform float dt;

//
// check bounce from a horizontal rectangle
//
bool checkBounce ( inout vec4 pos, inout vec4 vel, in float dt, in vec3 boxMin, in vec3 boxMax )
{
	float	h  = boxMin.z;		// z-coord of the rectangle
	float	dz = vel.z * dt;
	
	if ( (pos.z - h) * (pos.z + dz - h) > 0.0 )
		return false;			// no collision
		
	float	t  = (h - pos.z) / vel.z;	// time of collision
	vec3	pc = pos.xyz + t * vel.xyz;	// poin of collision with z plane
	
	if ( !all(lessThan((pc.xy - boxMin.xy)*(pc.xy - boxMax.xy), vec2 ( 0.0 ))))
		return false;					// collision point out of rect
		
	vel.z = -vel.z;						// bounce off
	vel  *= 0.9;						// dumping
	
	pos.xyz = pc + (dt - t) * vel.xyz;
	
	return true;
}

void main ()
{
	const vec3 box1Min = vec3 ( -10.0, -10.0, -2.0  );
	const vec3 box1Max = vec3 (  10.0,  10.0, -2.0  );
	const vec3 box2Min = vec3 (  0.0,  0.0, -1.5  );
	const vec3 box2Max = vec3 (  2.0,  2.0, -1.5  );
	const vec3 box3Min = vec3 ( -2.0, -2.0, -1.0 );
	const vec3 box3Max = vec3 (  0.0,  0.0, -1.0 );
	const vec4 g       = vec4 (  0.0,  0.0, -1.0, 0.0 );
	
	vec4	pos = gl_Vertex;
	vec4	vel = gl_MultiTexCoord0 + dt * g;
	
	if ( !checkBounce ( pos, vel, dt, box1Min, box1Max ) )
		if ( !checkBounce ( pos, vel, dt, box2Min, box2Max ) )
			if ( !checkBounce ( pos, vel, dt, box3Min, box3Max ) )
				pos += vel * dt;

	newPos      = vec4 ( pos.xyz, 1.0 );
	newVel      = vel;
	gl_Position = pos;
}